#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <stdio.h>
#include <time.h>
#include <string.h>

#include "data.h"
#include "callbacks.h"
#include "interface.h"
#include "support.h"

extern GtkWidget *start_dialog;
extern GtkWidget *progress_dialog;
extern GtkWidget *done_dialog;

extern int end_time, start_time;

struct s_profile *active_profile = NULL;

static GtkTreeIter active_iter;
static GtkTreeModel *old_model = NULL;
static GtkTreeIter old_iter;

void
on_start_dialog_close                  (GtkDialog       *dialog,
                                        gpointer         user_data)
{
	/*printf("close\n");*/
}


void
on_start_dialog_response               (GtkDialog       *dialog,
                                        gint             response_id,
                                        gpointer         user_data)
{
	struct s_profile *item;
	GtkTreeIter iter;
	GtkTreeView *treeview;
	GtkTreeModel *model;
	gboolean done;
	
	update_profile();

	treeview = GTK_TREE_VIEW(lookup_widget(start_dialog, "profiles_treeview"));
	model = gtk_tree_view_get_model(treeview);

	done = !gtk_tree_model_get_iter_first(model, &iter);
	while (!done) {
		gtk_tree_model_get(model, &iter, 1, &item, -1);
		data_save(item);
		done = !gtk_tree_model_iter_next(model, &iter);
	}
	data_save(NULL);
	/*printf("response %d\n", response_id);*/
}

void
selection_changed (GtkTreeSelection *selection)
{
	GtkTreeModel *model = NULL;
	struct s_profile *item = NULL;
	char *name;
	GtkTextBuffer *buf = gtk_text_view_get_buffer (GTK_TEXT_VIEW
			(lookup_widget(start_dialog, "note_textview")));

	update_profile();

	if (old_model && active_profile) {
		gtk_list_store_set (GTK_LIST_STORE(old_model), &old_iter, 0, active_profile->name, -1);
	}
	
	if (gtk_tree_selection_get_selected (selection, &model, &active_iter)) {
		old_model = model;
		memcpy(&old_iter, &active_iter, sizeof(active_iter));

		gtk_tree_model_get(model, &active_iter, 1, &item, -1);
		/* printf("seleceted: %s\n", item->name); */

		active_profile = item;
		
		/* name */
		gtk_entry_set_text(GTK_ENTRY(lookup_widget(start_dialog, "name_entry")), item->name ? item->name : "");

		/* time */
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(lookup_widget(start_dialog, "hour_spinbutton")),
				data_get_hour(item));
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(lookup_widget(start_dialog, "minute_spinbutton")),
				data_get_minute(item));
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(lookup_widget(start_dialog, "second_spinbutton")),
				data_get_seconds(item));
		
		/* event start */
		gtk_entry_set_text(GTK_ENTRY(lookup_widget(start_dialog, "event_start_entry")),
				item->event[EV_START].cmd ? item->event[EV_START].cmd : "");
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(start_dialog, "event_start_checkbutton")),
				item->event[EV_START].enabled ? TRUE : FALSE);
		/* event done */
		gtk_entry_set_text(GTK_ENTRY(lookup_widget(start_dialog, "event_done_entry")),
				item->event[EV_DONE].cmd ? item->event[EV_DONE].cmd : "");
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(start_dialog, "event_done_checkbutton")),
				item->event[EV_DONE].enabled ? TRUE : FALSE);
		/* event stop */
		gtk_entry_set_text(GTK_ENTRY(lookup_widget(start_dialog, "event_abort_entry")),
				item->event[EV_ABORT].cmd ? item->event[EV_ABORT].cmd : "");
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(start_dialog, "event_abort_checkbutton")),
				item->event[EV_ABORT].enabled ? TRUE : FALSE);

		/* wmcdclock */
		gtk_entry_set_text(GTK_ENTRY(lookup_widget(start_dialog, "wmcdclock_fifo_entry")),
				item->wmcdclock_fifo);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(
					lookup_widget(start_dialog, "wmcdclock_enable_checkbutton")),
				item->wmcdclock_enabled ? TRUE : FALSE);
		
		/* note */
		gtk_text_buffer_set_text (buf, item->note ? item->note : "", -1);
		
	}
}


void
on_profiles_treeview_row_activated     (GtkTreeView     *treeview,
                                        GtkTreePath     *path,
                                        GtkTreeViewColumn *column,
                                        gpointer         user_data)
{
	gtk_dialog_response(GTK_DIALOG(start_dialog), GTK_RESPONSE_OK);
}

gboolean
on_timeout                             (gpointer data)
{
	static int old_time = 0, new_time = 0;
	gchar str[32];
	int remain_time;


	/*
	if (timed_out) {
		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(lookup_widget(progress_dialog, "progressbar")),
				timed_out == 3 || timed_out == 5);
		timed_out = (timed_out % 5) + 1;
	} else {
	*/
	new_time = time(NULL);
	if (new_time != old_time) {
		old_time = new_time;
		remain_time = end_time - new_time;
		
		g_sprintf(str, "%02d:%02d:%02d",
				remain_time / 3600,
				(unsigned) remain_time / 60 % 60,
				(unsigned) remain_time % 60);
		gtk_progress_bar_set_text(GTK_PROGRESS_BAR(lookup_widget(progress_dialog, "progressbar")),
				str);
		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(lookup_widget(progress_dialog, "progressbar")),
				(gdouble) (new_time - start_time) / (gdouble) (end_time - start_time));
		
		if (new_time >= end_time) {
			gtk_dialog_response(GTK_DIALOG(progress_dialog), GTK_RESPONSE_OK);
			return FALSE;
			/*
			gtk_label_set_markup(GTK_LABEL(lookup_widget(progress_dialog, "action_label")),
					_("<b>Časovač dočasoval!</b>"));
			gtk_button_set_label(GTK_BUTTON(lookup_widget(progress_dialog, "button")),
					"gtk-close");
			gtk_image_set_from_stock(GTK_IMAGE(lookup_widget(progress_dialog, "image")),
					"gtk-dialog-warning", GTK_ICON_SIZE_DIALOG);
			gtk_window_set_position(GTK_WINDOW(progress_dialog), GTK_WIN_POS_CENTER_ALWAYS);
			gtk_window_set_keep_above(GTK_WINDOW(progress_dialog), TRUE);
			gtk_window_present(GTK_WINDOW(progress_dialog));
			timed_out = 1;
			*/
		}
	}
	
	return TRUE;
}

gboolean
on_done_timeout                        (gpointer data)
{
	static int s;
	int seq[2][4] = {
		{ 0, 1, 1, 1 },
		{ 1, 1, 0, 1 }
	};
	static GtkWidget *img1 = NULL, *img2 = NULL;
	
	if (!img1)
		img1 = lookup_widget(done_dialog, "image1");
	if (!img2)
		img2 = lookup_widget(done_dialog, "image2");
		
	gtk_widget_set_sensitive(img1, seq[0][s]);
	gtk_widget_set_sensitive(img2, seq[1][s]);

	s = (s + 1) % 4;
	return TRUE;
}

void
on_done_dialog_show                    (GtkWidget       *widget,
                                        gpointer         user_data)
{
	gtk_label_set_text(GTK_LABEL(lookup_widget(done_dialog, "profile_label")), active_profile->name);
	g_timeout_add(120, on_done_timeout, NULL);
}

void
on_profiles_new_button_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{
	struct s_profile *item;
	int i;
	GtkListStore *store;
	GtkTreeIter iter;
	GtkTreeView *treeview;

	/* create data item */
	item = data_create();
	item->name = NULL;
	item->time = active_profile->time;
	for (i = 0; i < EV_MAX; i++) {
		item->event[i].cmd = strdup(active_profile->event[i].cmd);
		item->event[i].enabled = active_profile->event[i].enabled;
	}
	item->wmcdclock_enabled = active_profile->wmcdclock_enabled;
	item->wmcdclock_fifo = strdup(active_profile->wmcdclock_fifo);

	/* append item to treeview */
	treeview = GTK_TREE_VIEW(lookup_widget(start_dialog, "profiles_treeview"));
	store = GTK_LIST_STORE(gtk_tree_view_get_model(treeview));
	gtk_list_store_append (store, &iter);
	gtk_list_store_set (store, &iter, 0, item->name, 1, item, -1);

	/* focus */
	gtk_tree_view_set_cursor(treeview, gtk_tree_model_get_path(GTK_TREE_MODEL(store), &iter), NULL, FALSE);
	gtk_widget_grab_focus(lookup_widget(start_dialog, "name_entry"));
}


void
on_profiles_del_button_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkListStore *store;
	GtkTreeView *treeview;
	GtkTreePath *path;
	GtkTreeIter tmp_iter;

	treeview = GTK_TREE_VIEW(lookup_widget(start_dialog, "profiles_treeview"));
	store = GTK_LIST_STORE(gtk_tree_view_get_model(treeview));
	old_model = NULL;

	gtk_tree_view_get_cursor(treeview, &path, NULL);
	gtk_list_store_remove(store, &active_iter);
	gtk_tree_model_get_iter(GTK_TREE_MODEL(store), &active_iter, path);

	while(!gtk_tree_model_get_iter(GTK_TREE_MODEL(store), &tmp_iter, path))
		gtk_tree_path_prev(path);
	gtk_tree_view_set_cursor(treeview, path, NULL, FALSE);
}


/* vim:set encoding=utf8: */

void
on_name_entry_changed                  (GtkEditable     *editable,
                                        gpointer         user_data)
{
	GtkTreeModel *model = NULL;
	GtkEntry *entry;
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(lookup_widget(start_dialog, "profiles_treeview")));
	entry = GTK_ENTRY(lookup_widget(start_dialog, "name_entry"));
	gtk_list_store_set (GTK_LIST_STORE(model), &active_iter, 0, gtk_entry_get_text(entry), -1);

}

