/*
 * data.c
 * main data manipulations
 *
 * Copyright (c) 2005 by Josef Kufner <jk@myserver.cz>
 *
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "data.h"

#define DATA_FILE	".gtimeout"
#define DATA_BUF	1024

/*****************************************************************************
 ** Creates new item on end of list
 * Returns: borrowed refference to item
 */
struct s_profile *data_create() {
	struct s_profile *profile;

	/* create new profile */
	profile = (struct s_profile *) malloc(sizeof(struct s_profile));
	if (!profile) {
		perror(PACKAGE ": data_create");
	}
	bzero(profile, sizeof(struct s_profile));

	return profile;
}

/*****************************************************************************
 ** Gets hour
 * Returns: Integer
 */
int data_get_hour(struct s_profile *item) {
	return (item->time / 3600.0);
}

/*****************************************************************************
 ** Gets minute
 * Returns: Integer
 */
int data_get_minute(struct s_profile *item) {
	return (int)(item->time / 60.0) % 60;
}

/*****************************************************************************
 ** Gets second
 * Returns: Integer
 */
int data_get_seconds(struct s_profile *item) {
	return item->time % 60;
}

/*****************************************************************************
 ** Loads one data item
 * Returns: reference to profile, must be freed
 */
struct s_profile *data_load () {
	struct s_profile *item = NULL;
	char *name = NULL;
	static FILE *f = NULL;
	char buf[DATA_BUF];
	int line = -1;
	
	if (!f) {
		name = (char *) malloc(strlen(getenv("HOME")) + strlen(DATA_FILE) + 3);
		strcpy(name, getenv("HOME"));
		strcat(name, "/");
		strcat(name, DATA_FILE);
		f = fopen(name, "r");
	}
	
	if (f && ! feof(f)) {
		while (!feof(f)) {
			fgets(buf, DATA_BUF, f);
			if (buf[strlen(buf) - 1] == '\n') {
				buf[strlen(buf) - 1] = '\0';
			}

			/* end of profile */
			if (buf[0] == '\0') {
				break;
			}
				
			if (buf[0] == '>') {
				item = data_create();
				item->name = strdup(buf + 1);
				line = 1;
			} else if (item && buf[0] == '\t' && line > 0) {
				switch (line) {
					case 1: /* time */
						item->time = atoi(buf + 1);
						break;
					case EV_START + 2: /* start event */
					case EV_DONE  + 2: /* stop envent */
					case EV_ABORT + 2: /* done event */
						item->event[line - 2].enabled = (buf[1] == '+');
						item->event[line - 2].cmd = strdup(buf + 2);
						break;
					case 5: /* wmcdclock */
						item->wmcdclock_enabled = (buf[1] == '+');
						item->wmcdclock_fifo = strdup(buf + 2);
						break;
					default:
						/* shit */
						break;
				}
				line ++;
			}
		}
	}

	if (feof(f)) {	
		fclose(f);
		f = NULL;
	}

	return item;

#if 0
		/* Defaults */
		item = data_create();
		item->name = strdup("One minute");
		item->time = 60;
		item->note = "This is test item, one minute test item. If you can see this, "
			     "your data is not loaded :)";
		item->wmcdclock_enabled = 1;
		item->wmcdclock_fifo =  strdup("~/.wmcdclock.fifo");

		item = data_create();
		item->name = strdup("One hour");
		item->time = 3600;
		item->note = "Another test item...";
		item->wmcdclock_enabled = 1;
		item->wmcdclock_fifo =  strdup("~/.wmcdclock.fifo");

		item = data_create();
		item->name = strdup("Five seconds");
		item->time = 5;
		item->note = "This is test item, five seconds test item. It is good for testing events.";
		item->wmcdclock_enabled = 1;
		item->wmcdclock_fifo =  strdup("~/.wmcdclock.fifo");

		item = data_create();
		item->name = strdup("Pizza");
		item->time = 765;
		item->note = "Pizza !";
		item->wmcdclock_enabled = 1;
		item->wmcdclock_fifo =  strdup("~/.wmcdclock.fifo");
		
		return 1;
#endif
}

/*****************************************************************************
 ** Saves data item
 * Returns: non-zero on error
 */
int data_save (struct s_profile *item) {
	static FILE *f_cfg = NULL;
	char *name;

	if (!f_cfg && item) {
		name = (char *) malloc(strlen(getenv("HOME")) + strlen(DATA_FILE) + 3);
		strcpy(name, getenv("HOME"));
		strcat(name, "/");
		strcat(name, DATA_FILE);
		
		if ((f_cfg = fopen(name, "w")))
			fprintf(f_cfg, "gTimeOut cofiguration file (cfg_ver=2)\n");
	}

	if (f_cfg && item) {
		fprintf(f_cfg, ">%s\n", item->name);
		fprintf(f_cfg, "\t%ld\n", item->time);
		fprintf(f_cfg, "\t%c%s\n", item->event[EV_START].enabled ? '+' : '-', item->event[EV_START].cmd);
		fprintf(f_cfg, "\t%c%s\n", item->event[EV_DONE].enabled ? '+' : '-', item->event[EV_DONE].cmd);
		fprintf(f_cfg, "\t%c%s\n", item->event[EV_ABORT].enabled ? '+' : '-', item->event[EV_ABORT].cmd);
		fprintf(f_cfg, "\t%c%s\n", item->wmcdclock_enabled ? '+' : '-', item->wmcdclock_fifo);
		fprintf(f_cfg, "\n");
		return 1;
	}
	
	if (f_cfg && !item)
		fclose(f_cfg);
	
	return 0;
}

/*****************************************************************************
 ** Total cleanup
 */
void data_destroy (struct s_profile *item) {

	/* TODO free all */
	
	if (item)
		free(item);
}

