#include <gtk/gtk.h>
#include <stdio.h>
#include <time.h>
#include <string.h>
#include "callbacks.h"
#include "data.h"

extern GtkWidget *start_dialog;
extern GtkWidget *progress_dialog;

extern int end_time, start_time;

void fill_start_dialog (void) {
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	GtkTreeView *treeview = GTK_TREE_VIEW(lookup_widget(start_dialog, "profiles_treeview"));
	GtkTreeModel *model = gtk_tree_view_get_model (treeview);
	GtkTreeSelection *selection;
	struct s_profile *item;
	GtkListStore *store;
	GtkTreeIter iter;

	/* create list store */
	store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_POINTER);

	/* add data to the list store */
	while ((item = data_load())){
		gtk_list_store_append (store, &iter);
		gtk_list_store_set (store, &iter, 0, item->name, 1, item, -1);
	}

	gtk_tree_view_set_model(treeview, GTK_TREE_MODEL(store));
	
	/* column for description */
	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes ("Profil",
							   renderer,
							   "text",
							   0,
							   NULL);
	gtk_tree_view_column_set_sort_column_id (column, 0);
	gtk_tree_view_append_column (treeview, column);
	
	selection = gtk_tree_view_get_selection(treeview);
	gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);

	g_signal_connect (selection, "changed", G_CALLBACK (selection_changed), NULL);
}

void start_timer (int t, char *name) {
	printf("Timer: %d sec, \"%s\"\n", t, name);
	start_time = time(NULL);
	end_time = start_time + t;

	g_timeout_add(100, on_timeout, NULL);
	
	gtk_label_set_text(GTK_LABEL(lookup_widget(progress_dialog, "profile_label")), name);
}

void update_profile(void) {
	char *text;
	int h, m, s;

	if (active_profile && start_dialog) {
		/* name */
		text = (char *) gtk_entry_get_text(GTK_ENTRY(lookup_widget(start_dialog, "name_entry")));
		if (active_profile->name)
			free(active_profile->name);
		active_profile->name = strdup(text);

		/* time */
		h = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(lookup_widget(start_dialog,
						"hour_spinbutton")));
		m = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(lookup_widget(start_dialog, 
						"minute_spinbutton")));
		s = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(lookup_widget(start_dialog, 
						"second_spinbutton")));
		active_profile->time = 3600 * h + 60 * m + s;

		/* events */
		active_profile->event[EV_START].enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
					lookup_widget(start_dialog, "event_start_checkbutton")));
		if (active_profile->event[EV_START].cmd)
			free(active_profile->event[EV_START].cmd);
		active_profile->event[EV_START].cmd = strdup(gtk_entry_get_text(GTK_ENTRY(
					lookup_widget(start_dialog, "event_start_entry"))));

		active_profile->event[EV_DONE].enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
					lookup_widget(start_dialog, "event_done_checkbutton")));
		if (active_profile->event[EV_DONE].cmd)
			free(active_profile->event[EV_DONE].cmd);
		active_profile->event[EV_DONE].cmd = strdup(gtk_entry_get_text(GTK_ENTRY(
					lookup_widget(start_dialog, "event_done_entry"))));

		active_profile->event[EV_ABORT].enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
					lookup_widget(start_dialog, "event_abort_checkbutton")));
		if (active_profile->event[EV_ABORT].cmd)
			free(active_profile->event[EV_ABORT].cmd);
		active_profile->event[EV_ABORT].cmd = strdup(gtk_entry_get_text(GTK_ENTRY(
					lookup_widget(start_dialog, "event_abort_entry"))));

		/* wmcdclock */
		active_profile->wmcdclock_enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
					lookup_widget(start_dialog, "wmcdclock_enable_checkbutton")));
		if (active_profile->wmcdclock_fifo)
			free(active_profile->wmcdclock_fifo);
		active_profile->wmcdclock_fifo = strdup(gtk_entry_get_text(GTK_ENTRY(
					lookup_widget(start_dialog, "wmcdclock_fifo_entry"))));

	}
}

